# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <math.h>
# include "decls.h"

# define C 158000. /* speed of sound in cm/sec */
# define TWOPI 6.28318530717958647688
# define TRUE 1
# define FALSE 0
# define NPTCLS	2
# define NFILT 8192
# define RANDNORM 32768.

struct particle {
	double xrange;
	double yrange;
	double ampl;
} ptcls[NPTCLS];

double rayleigh(void);
int compar(struct particle *p1,struct particle *p2);
int isfile(char *root,int ext,char *filename);
double doamp(struct particle *p, double range, int *past);

/*
This routine simulates the signal from one group of particles.
It is assmued that the beam pattern is uniform over its extent.
One velocity is present in the sample volume.
The scattering coefficients from each particle are rayleigh distributed.
Note: when changing from positive to negative velocity the following must
be changed:

	1) ptcls[i].xrange
	2) compar()
*/



main(argc,argv)
int argc;
char *argv[];
{
	double t,f0,omega0,theta,costheta,sintheta,tau0,sr,sg,sgcos,sgsin;
	double sgincr;
	double pulsetime;
	double ttotal,dt,dtout;
	double amp;
	double a1,a2,a3,a4;
	double lambda;

	double range=0.;
	double arg;
	double vax,vt;
	double k;
	double R1 = 2.;
	double R2 = 2.;

	float sgf;
	struct particle p1;
	int it;
	long ittotal;
	int i,n,ifile;
	int up,down;
	FILE *fpout;
	FILE *fprcd;
	FILE *fpnfo;
	int iout,io;
	int nzeros,nf,nnf;
	char filename[200];
 	char fileconv[200];
 	char filebur[200];
 	char ibuf[5];
	int (*comp)();
	int ipstart;
	int past;
	int printbur;
	float s1,c1;

	if((fprcd = fopen("simpsim.rcd","a")) == NULL){
		fprintf(stderr,"error: %s cannot open file %s\n",
			argv[0],"simpsim.rcd");
		exit(-1);
	}
	if((fpnfo = fopen("simpsim.nfo","a")) == NULL){
		fprintf(stderr,"error: %s cannot open file %s\n",
			argv[0],"simpsim.nfo");
		exit(-1);
	}
	ifile = 1;
	while(isfile("simpsim",ifile,filename)) ifile++;
	if((fpout = fopen(filename,"wb")) == NULL){
		fprintf(stderr,"error: %s cannot open file %s\n",
			argv[0],filename);
		exit(-1);
	}
 	strcpy(filebur,"simpsim"); /* burst file */
 	sprintf(ibuf,".%03d",ifile);
 	strcat(filebur,ibuf);
 	strcpy(fileconv,"simpsim");
 	sprintf(ibuf,".%03d",ifile);
 	strcat(fileconv,ibuf);
	comp = compar;
/* Read in the filtering coefficients. */
	ttotal = .16; /* 16 msec simulation */
	tau0 = 1./32000.; /* pulse repetition time */
	dt = tau0; /* 8192 samples per pulse */
	f0 = 10000000; /* 10 MHz carrier frequency */
	pulsetime = 1.e-6; /* 10 cycles/pulse */
	omega0 = TWOPI*f0;
	k = omega0/C;
	theta = TWOPI*0./360.; /* 60 degree receiver angle */
	costheta = cos(theta);
	sintheta = sin(theta);
	vax = 60.; /* 60 cm/sec particle velocity */
	vt = 0.;
	range = 0.; /* Initial Range 0 cm */
	lambda = TWOPI*C/omega0;
	printf("lambda is %lf\n",lambda);

	ittotal = ttotal/dt;
	printf("ittotal is %ld\n",ittotal);
	fprintf(fprcd,"ittotal %ld, dt %lg, f0 %lg, ttotal %lf, tau0 %lf\n",
	ittotal, dt, f0, ttotal, tau0);
	fclose(fprcd);

/* generate the particles */
	srand(3482);
/*
	for(i = 0; i < NPTCLS; i++){
		ptcls[i].xrange = 2.*(double)rand()/RANDNORM - 0.1;
		ptcls[i].yrange = 0.1*(double)rand()/RANDNORM - 0.05;
		ptcls[i].ampl = rayleigh();
	}
*/
		ptcls[0].xrange = 0.2;
		ptcls[0].yrange = 0.0;
		ptcls[0].ampl = rayleigh();

		ptcls[1].xrange = 0.2 + lambda/2.;
		ptcls[1].yrange = 0.0;
		ptcls[1].ampl = rayleigh();

	qsort(ptcls,NPTCLS,sizeof(struct particle),comp);
	for(i = 0; i < NPTCLS; i++){
		printf("xrange %lf, yrange %lf, ampl %lf\n",
			ptcls[i].xrange, ptcls[i].yrange, ptcls[i].ampl);
	}
	fflush(stdout);
	printf("ptcls[0].xrange %lf, ptcls[1].xrange %lf\n",
		ptcls[0].xrange, ptcls[1].xrange);
	t = 0.;
	iout = NFILT/64;
	dtout = dt*(double)iout;
	if(ifile == 1){
		fprintf(fpnfo,"<<Atod Information>>\n");
		fprintf(fpnfo,"File #   Samples   Sample Rate    ");
		fprintf(fpnfo,"Date    # Chan    Key");
		fprintf(fpnfo,"   gain1   gain2   gain3   gain4\n");
	}
	fprintf(fpnfo,"%03d\t%d\t%lf\tFeb  12 1996  2   CS 0 0 0 0\n",ifile,
		ittotal/iout,1./dtout);
	fclose(fpnfo);
	ipstart = 0;
	past = 0;
	printf("start main loop\n");
	for(it = 0; it < ittotal; it++){ /* loop over time */
		if(it%100 == 0) printf("iteration %d\n",it);
		t = (double)it*dt;
		sgcos = 0.;
		sgsin = 0.;
		for(i = ipstart; i < NPTCLS; i++){
			range = ptcls[i].xrange - vax*t;
			amp = doamp(&ptcls[i],range,&past);
			if(past){ /* this particle has passed the samp. vol */
				past = 1;
				ipstart++;
				continue;
			}

/* calculate downmixed signals */
			a1 = R1 - range;
			a2 = ptcls[i].yrange;
			a3 = R2*costheta - range;
			a4 = R2*sintheta - ptcls[i].yrange;
			arg = k*(sqrt(a1*a1+a2*a2) + sqrt(a3*a3+a4*a4));
			sgcos += amp*cos(arg);
			sgsin += amp*sin(arg);
		}

/* Write the data */
		s1 = sgcos;
		c1 = sgsin;
		fwrite(&s1,4,1,fpout);
		fwrite(&c1,4,1,fpout);


	}
	return(0);

}


double
doamp(struct particle *p, double range, int *past)
{
	double x,amp;

	*past = 0;
	if(range < 0) *past = 1;
	if(range > 0.1 || range < 0) return(0.);
	x = 0.05 - fabs(range-0.05);
	amp = 20.*x; /* amplitude linear with particle position */;
/*	amp = 20.; */
	printf("xrange, amp is %lf, range is %lf, x %lf\n",p->xrange,amp,range,x);
	fflush(stdout);

	return(amp);
	
}


# include <sys/stat.h>
# include <sys/types.h>

int
isfile(root,ext,filename)
char *root,*filename;
int ext;
{
	char extstr[100];
	struct stat statbuf;
	int sret;

	sprintf(extstr,".%03d",ext);
	strcpy(filename,root);
	strcat(filename,extstr);

	if((sret = stat(filename,&statbuf))==-1){
		return(0);
	} else {
		return(1);
	}
}
compar(p1,p2)
struct particle *p1,*p2;
{
	if( p1->xrange < p2->xrange) return(-1);
	else if( p1->xrange > p2->xrange) return(1);
	else return(0);
}


/*
Returns a Rayleigh distributed random variable.
*/
double
rayleigh()
{
	double v1,v2,r,fac1,fac,ray;

	do{
		v1 = (double)rand()/RANDNORM - 1.;
		v2 = (double)rand()/RANDNORM - 1.;
		r = v1*v1 + v2*v2;
	} while(r >= 1.0 || r == 0.0);
	fac = sqrt(-2.0*log(r)/r);
	v1 = v1*fac;
	v2 = v2*fac;
	ray = sqrt(v1*v1 + v2*v2);
	return(ray);
}
