# include <dos.h>
# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <math.h>
# include "decls.h"

# define C 158000. /* speed of sound in cm/sec */
# define TWOPI 6.28318530717958647688
# define TRUE 1
# define FALSE 0
# define NPTCLS	20000000 /* reduced from 20,000,000 */
# define NFILT 8192
# define RANDNORM 32768.
# define NCHUNK	1000

struct particle {
	float xrange;
	float yrange;
	float ampl;
};

double rayleigh(void);
int compar(struct particle *p1,struct particle *p2);
long isfile(char *root,int ext,char *filename);
double doamp(double range, long *past);
void getdate(struct date *datep);

/*
This routine simulates the signal from one group of particles.
It is assmued that the beam pattern is uniform over its extent.
One velocity is present in the sample volume.
The scattering coefficients from each particle are rayleigh distributed.
Note: when changing from positive to negative velocity the following must
be changed:

	1) ptcls[i].xrange
	2) compar()
*/



main(argc,argv)
long argc;
char *argv[];
{
	double t,f0,omega0,theta,costheta,sintheta,tau0,sr,sg,sgcos,sgsin;
	double angle;
	double sgincr;
	double pulsetime;
	double ttotal,dt,dtout;
	double amp;
	double a1,a2,a3,a4;
	double lambda;

	double range=0.;
	double arg;
	double vax,vt;
	double k;
	double R1 = 2.;
	double R2 = 2.;

	float sgf;
	struct particle p1;
	long it;
	long ittotal;
	long i,j,n,ifile;
	long up,down;
	FILE *fpout;
	FILE *fprcd;
	FILE *fpnfo;
	long iout,io;
	long nzeros,nf,nnf;
	char filename[200];
 	char fileconv[200];
 	char filebur[200];
 	char ibuf[5];
	int (*comp)();
	long ipstart;
	long past;
	long printbur;
	float s1,c1;
	int n_in;
	double totalrange; /* 20 cm */
	int nchunks;
	double therange;
	double drange;
	struct particle *ptcls,*part;
	struct date *today;

	ptcls = (struct particle *)malloc(sizeof(struct particle)*NPTCLS);
	part = (struct particle *)malloc(sizeof(struct particle)*NCHUNK);

	angle = 30; /* 30 degree receiver probe angle */
	if(argc == 2){
		angle = atof(argv[1]);
	}
	if((fprcd = fopen("simpsim1.rcd","a")) == NULL){
		fprintf(stderr,"error: %s cannot open file %s\n",
			argv[0],"simpsim.rcd");
		exit(-1);
	}
	if((fpnfo = fopen("simpsim1.nfo","a")) == NULL){
		fprintf(stderr,"error: %s cannot open file %s\n",
			argv[0],"simpsim.nfo");
		exit(-1);
	}
	ifile = 1;
	while(isfile("simpsim1",ifile,filename)) ifile++;
	if((fpout = fopen(filename,"wb")) == NULL){
		fprintf(stderr,"error: %s cannot open file %s\n",
			argv[0],filename);
		exit(-1);
	}
 	strcpy(filebur,"simpsim"); /* burst file */
 	sprintf(ibuf,".%03ld",ifile);
 	strcat(filebur,ibuf);
 	strcpy(fileconv,"simpsim");
 	sprintf(ibuf,".%03ld",ifile);
 	strcat(fileconv,ibuf);
	comp = compar;

	ttotal = 2.16; /* 2.16 sec simulation */
	tau0 = 1./32000.; /* pulse repetition time */
	dt = tau0; /* 8192 samples per pulse */
	f0 = 10000000; /* 10 MHz carrier frequency */
	pulsetime = 1.e-6; /* 10 cycles/pulse */
	omega0 = TWOPI*f0;
	k = omega0/C;
	theta = TWOPI*angle/360.; /* 31 degree receiver angle */
	costheta = cos(theta);
	sintheta = sin(theta);
	vax = 60.; /* 60 cm/sec particle velocity */
	vt = 0.;
	range = 0.; /* Initial Range 0 cm */
	lambda = TWOPI*C/omega0;
	printf("lambda is %lf\n",lambda);

	ittotal = ttotal/dt;
	printf("ittotal is %ld\n",ittotal);
	printf("FILE fprcd is %ld\n",fprcd);
	if(ifile == 1){
		fprintf(fprcd,"file\tittotal\tdt\t\tf0\tttotal\t\ttau0\tangle\n");
	}
	
	printf("%ld\t%ld\t%lg\t%lg\t%lf\t%lf\t%lf\n",
		ifile,ittotal, dt, f0, ttotal, tau0, angle);
	fprintf(fprcd,"%ld\t%ld\t%lg\t%lg\t%lf\t%lf\t%lf\n",
		ifile,ittotal, dt, f0, ttotal, tau0, angle);
	fflush(fprcd);
	fclose(fprcd);

/* generate the particles in chunks of NCHUNK */
	srand(3482);
	totalrange = vax*ttotal; /* 20 cm */
	nchunks = NPTCLS/NCHUNK;
	therange = 0.;
	drange = totalrange/nchunks;
	for(j = 0; j < nchunks; j++){
		for(i = 0; i < NCHUNK; i++){
			part[i].xrange = therange + drange*(double)rand()/RANDNORM;
			part[i].yrange = 0.1*(double)rand()/RANDNORM - 0.05;
			part[i].ampl = rayleigh();
		}
		if(j%1000==0){
			printf("sorting %ld of %ld\n",j,nchunks);
			fflush(stdout);
		}

		qsort(part,NCHUNK,sizeof(struct particle),comp);
		for(i = 0; i < NCHUNK; i++){
			ptcls[j*NCHUNK+i].xrange = part[i].xrange;
			ptcls[j*NCHUNK+i].yrange = part[i].yrange;
			ptcls[j*NCHUNK+i].ampl = part[i].ampl;
		}
		therange += drange;
	}
	printf("sorted and rearranged into ptcls array.\n");
/*
	for(i = 0; i < 100; i++){
		printf("%ld: %lf %lf\n",i,ptcls[i].xrange,ptcls[i].yrange);
	}
	for(i = NPTCLS-100; i < NPTCLS; i++){
		printf("%ld: %lf %lf\n",i,ptcls[i].xrange,ptcls[i].yrange);
	}
*/
	fflush(stdout);
	t = 0.;
	iout = NFILT/64;
	dtout = dt*(double)iout;
	printf("FILE fpnfo is %ld\n",fpnfo);
	if(ifile == 1){
		fprintf(fpnfo,"<<Atod Information>>\n");
		fprintf(fpnfo,"File #   Samples   Sample Rate    ");
		fprintf(fpnfo,"Date    # Chan    Key");
		fprintf(fpnfo,"   gain1   gain2   gain3   gain4\n");
	}

/*
	getdate(today);
	fprintf(fpnfo,"%03ld\t%d\t%lf\t%d/%d/%d\t2   CS 0 0 0 0\n",ifile,
		ittotal,1./dt,today->da_mon,today->da_day,today->da_year);
	fclose(fpnfo);
*/
	fprintf(fpnfo,"%03ld\t%ld\t%lf\tDec 27 2000\t2   CS 0 0 0 0\t%lf\n",ifile,
		ittotal,1./dt,angle);
	fclose(fpnfo);
	ipstart = 0;
	past = 0;
	printf("start main loop\n");
	fflush(stdout);
	n_in = 0;
	printf("ittotal is %ld, ifile %ld, angle %lf\n",ittotal,ifile,angle);
	for(it = 0; it < ittotal; it++){ /* loop over time */
		t = (double)it*dt;
		sgcos = 0.;
		sgsin = 0.;
		if(it%1000 == 0){
			printf("time step %ld, time %lf n_in %ld\n",it,t,n_in);
			fflush(stdout);
		}
		n_in = 0;
		for(i = ipstart; i < NPTCLS; i++){
			range = ptcls[i].xrange - vax*t;
			amp = doamp(range,&past);
			if(past == 1){ /* this particle has passed the samp. vol */
				past = 0;
				ipstart++;
				continue;
			}
			if(past == -1){
				past = 0;
				break;
			}
			if(amp == 0 && !past){
				continue;
			}

/* calculate downmixed signals */
			n_in++;
			a1 = R1 - range;
			a2 = ptcls[i].yrange;
			a3 = R2*costheta - range;
			a4 = R2*sintheta - ptcls[i].yrange;
			arg = k*(sqrt(a1*a1+a2*a2) + sqrt(a3*a3+a4*a4));
			sgcos += amp*cos(arg);
			sgsin += amp*sin(arg);
		}

/* Write the data */
		s1 = sgcos;
		c1 = sgsin;
		fwrite(&s1,4,1,fpout);
		fwrite(&c1,4,1,fpout);
		if(n_in == 0){
			printf("no more particles in sample volume\n");
			printf("ipstart is %ld\n",ipstart);
			break;
		}

	}
	printf("returning after %ld time steps\n",it);
	return(0);

}


double
doamp(double range, long *past)
{
	double x,amp;

	*past = 0;
	if(range < 0) *past = 1; /* particle has gone past sample volume */
	if(range > 0.1) *past = -1; /* particle hasn't reached sample vol. */
	if(range > 0.1 || range < 0) return(0.);
	x = 0.05 - fabs(range-0.05);
	amp = 20.*x; /* amplitude linear with particle position */;
/*	amp = 20.; */

	return(amp);
	
}


# include <sys/stat.h>
# include <sys/types.h>

long
isfile(root,ext,filename)
char *root,*filename;
int ext;
{
	char extstr[100];
	struct stat statbuf;
	long sret;

	sprintf(extstr,".%03d",ext);
	strcpy(filename,root);
	strcat(filename,extstr);

	if((sret = stat(filename,&statbuf))==-1){
		return(0);
	} else {
		return(1);
	}
}
compar(p1,p2)
struct particle *p1,*p2;
{
	if( p1->xrange < p2->xrange) return(-1);
	else if( p1->xrange > p2->xrange) return(1);
	else return(0);
}


/*
Returns a Rayleigh distributed random variable.
*/
double
rayleigh()
{
	double v1,v2,r,fac1,fac,ray;

	do{
		v1 = (double)rand()/RANDNORM - 1.;
		v2 = (double)rand()/RANDNORM - 1.;
		r = v1*v1 + v2*v2;
	} while(r >= 1.0 || r == 0.0);
	fac = sqrt(-2.0*log(r)/r);
	v1 = v1*fac;
	v2 = v2*fac;
	ray = sqrt(v1*v1 + v2*v2);
	return(ray);
}



