/*
Get the references from a tex file.  This looks for the character
`[` followed by numbers separated by commas, followed by ']'.  If
it finds an illegal character before it finds the ']', it assumes it
is not working on a reference, and moves on.

The references are stored in an array such that the array position
is the number arbitrarily assigned to the reference in the database,
and the number in the array is the position of the reference in the
manuscript file.
*/


# define NREFS	800
# define FROMBEG 0
# include <stdio.h>
# include <stdlib.h>
# include <string.h>

long refs[NREFS];
long positions[NREFS]; /* positions of the references in the database file */
long nrefs;
FILE *dbfile;
FILE *newpaper;

int getref();
int hashdb();
void sortdb();
int compar();
int rmext();

main(argc,argv)
int argc;
char *argv[];
{
	int i;
	char infname[100];
	int c;
	int nrdb;

	if(argc != 2){
		fprintf(stderr,"usage: %s databasefile\n",argv[0]);
		exit(-1);
	}
	if( (newpaper = fopen("new.tex","w")) == NULL){
		fprintf(stderr,"error: %s cannot open output file %s\n",
			argv[0],"new.tex");
	}
	strcpy(infname,argv[1]);
	nrefs = 0;
	for(i = 0; i < NREFS; i++){
		refs[i] = 0;
	}

/*
Find the order of the references in the text, and change the numbers
in the text.
*/
	while( (c = getchar()) != EOF){
		if(c == '[') getref();
		else(putc(c,newpaper));
	}
	fprintf(stderr,"%d total unique references\n",nrefs);
	nrdb = hashdb(infname);
	printf("Database hash\n");
	for(i = 0; i < nrdb; i++){
		printf("%d %ld\n",i,positions[i]);
	}
	sortdb(infname);
	return(0);
}

getref()
{
	int ref;
	int c;
	int sf;
	int i,j;
	char hold[200];
	char numhold[80];
	int ihold;
	int newrefs[50];
	int holdrefs[50];
	int nholdrefs;
	int nnewrefs;
	int compar();
	int (*comp)();

	comp = compar;

	nnewrefs = 0;
	nholdrefs = 0;
	ihold = 0;
	hold[ihold++] = '[';
	for(;;){
		
		sf = scanf("%d",&ref);
		if(sf == 1){
			sprintf(numhold,"%d",ref);
			hold[ihold] = '\0';
			strcat(hold,numhold);
			ihold += strlen(numhold);
		}
		if(sf != 1){ /* false alarm, reset */
			hold[ihold] = '\0';
/*			fprintf(newpaper,"%s",hold);*/
			for(i = 0; i < nholdrefs; i++){
				refs[nholdrefs] = 0;
			}
			nrefs -= nholdrefs;
			return(0);
		}
		if(refs[ref] == 0){
			nrefs++;
			refs[ref] = nrefs;
			holdrefs[nholdrefs] = ref;
			nholdrefs++;

			printf("%d\n",ref);
			newrefs[nnewrefs] = nrefs;
		} else {
			newrefs[nnewrefs] = refs[ref];
		}
		nnewrefs++;
/*
Keep track of each character in case this is a false alarm,
(i.e. in case this is just a '[' from an equation instead of
a reference.
*/
		while( (hold[ihold++] = c = getchar()) == ',' || c == ' '
				|| c == '\n');
		
		if( c == ']'){
/*
Sort the references so that they are numbered consequatively.
*/
			qsort(newrefs,nnewrefs,sizeof(int),comp);
/*			fprintf(newpaper,"${}^{%d",newrefs[0]);*/
			fprintf(newpaper,"[%d",newrefs[0]);
			newrefs[nnewrefs] = -1;
			for(i = 1; i < nnewrefs; i++){
				j = 0;
/*
Check for adjacent references that must be written as 234-237 instead
of 234, 235, 236, 237.
*/
				while(newrefs[i] == newrefs[i-1] + 1){
					i++;
					j++;
				}

				if( j > 1) fprintf(newpaper,"-");
				else fprintf(newpaper,", ");

				if( j > 0 ) i--;

				fprintf(newpaper,"%d",newrefs[i]);

			}

/*			fprintf(newpaper,"}$"); */
			fprintf(newpaper,"]");

			return(nrefs);
		}
		else if( c >= '0' && c <= '9') ungetc(c,stdin);
		else{ /* false alarm, reset */
			hold[ihold] = '\0';
			fprintf(newpaper,"%s",hold);
			fprintf(stderr,"putting back %d\n",ref);
			for(i = 0; i < nholdrefs; i++){
				refs[holdrefs[i]] = 0;
			}
			nrefs -= nholdrefs;
			return(0);
		}
	}
}

/*
This routine looks at the bibliography database file and determines
the file positions at which each bibliography entry begins.
*/
hashdb(infname)
char infname[];
{
	int i;
	int c;
	int iline;
	int nrdb; /* number of references in the database */
	char line[80];
	int refno;
	long position;

/*
The file is opened up in binary mode so that the program correctly
counts <cr><lf> as two characters.
*/
	if( (dbfile = fopen(infname,"rb") ) == NULL){
		fprintf(stderr,"error: cannot open file %s\n",
			"hasdb",infname);
		exit(-1);
	}
	position = 0;
	nrdb = 0;
	for(;;){
		while( ( c = getc(dbfile) ) != '%'){
			position++;
			if( c == EOF ){
				return(nrdb);
			}
		}
		position++;
		if( ( c = getc(dbfile) ) == 'Z'){
			iline = 0;
			while( (line[iline++] = getc(dbfile)) != '\n');
			line[iline] = '\0';
			sscanf(line,"%d",&refno);
			if(refno != nrdb + 1){
				fprintf(stderr,
		"error: Z number %d does not match entry order %d\n",
		refno,nrdb+1);
			}
			positions[nrdb + 1] = position - 1;
			nrdb++;
			position += iline;
		}
		position++;
		
	}
}

/*
Sort the database according to the refs array.  This creates a new
database file which has all the entries sorted according to their
order in the manuscript.
*/
void
sortdb(infname)
char infname[];
{
	int j;
	int i;
	long nchars;
	char areference[50000];
	FILE *newdbfile;
	char outfname[100];
	int irefs[NREFS];

	strcpy(outfname,infname);
	rmext(outfname);
	strcat(outfname,".hsh");
	if( (newdbfile = fopen(outfname,"w") ) == NULL){
		fprintf(stderr,"error: %s cannot open file %s\n",
			"sortdb",outfname);
		exit(-1);
	}
/*
Invert the refs array.  For the array refs[], the index is the (Z-)number
in the manuscript file, and the value is the number in the final
bibliography.  For the array irefs[], the index is the number in the
final bibliography, and the value is the (Z-)number in the manuscript file.
*/
	for(i = 1; i < NREFS; i++){
			irefs[i] = 0;
	}
	for(i = 1; i < NREFS; i++){
		if(refs[i] != 0){
			irefs[refs[i]] = i;
		}
	}
	for(i = 1; i < NREFS; i++){
		if(irefs[i] != 0){
			fseek(dbfile,positions[irefs[i]],FROMBEG);
			nchars = positions[irefs[i] + 1] - positions[irefs[i]];
			fread(areference,1,nchars,dbfile);
			fwrite(areference,1,nchars,newdbfile);
		}
	}
}

compar(el1,el2)
int *el1,*el2;
{
	if( *el1 > *el2) return(1);
	else if( *el1 < *el2) return(-1);
	else return(0);
}

/* remove the extension from a file name */
rmext(char *str)
{
	char *strp;
	char c;
	
	strp = str;
	while( (c = *strp++) != '.' && c != '\0');
	if(c == '.'){
		strp--;
		*strp = '\0';
		return(1); /* extension successfully removed */
	} else {
		return(0); /* no extension in the string */
	}
}
