# include <stdio.h>
# define AUTHOR	0
# define TITLE	1
# define SOURCE	2
# define ABSTRACT	3
# define NUMBER		4
# define WHERE		5
# define YEAR		6
# define TRUE 1
# define FALSE 0
# define COLONLOC	13
# define LINEBEG	16
/*
   RECORD NO.:  3399011 INSPEC Abstract No: A89078449
       AUTHOR:  Gilson, W.; Orphanoudakis, S.
       EDITOR:  Harris, G.; Walker, C.
  CORP SOURCE:  Lincoln Lab., MIT, Lexington, MA, USA
        TITLE:  Error bounds for wide-band high resolution Doppler 
                ultrasound blood flow measurement
       SOURCE:  Proceedings of the Annual International Conference of the 
                IEEE Engineering in Medicine and Biology Society (IEEE Cat. 
                No.88CH2566-8), p. 4 vol. (xxxiv+xxiii+xix+xix+1937), 473-4
PLACE OF PUBL:  USA
     LANGUAGE:  English
    PUBLISHER:  IEEE; New York, NY, USA
  SPONSOR ORG:  IEEE
CONF LOCATION:  New Orleans, LA, USA; 4-7 Nov. 1988
         YEAR:  1988
 COPYRIGHT NO:  CH2566-8/88/0000-0473$01.00
    TREATMENT:  T Theoretical or Mathematical
     ABSTRACT:  Cramer-Rao bounds on the RMS error of Doppler flow 
*/
char *lineflags[14] = {"       AUTHOR","        TITLE","       SOURCE",
"     ABSTRACT","   RECORD NO.", "  CORP SOURCE","         YEAR"};
int nflags = 7;

/* definitions for source field */
char *kwd[5] = {"vol.","no.","p."};
int nkwds = 3;
# define TEXTWORD	-1
# define VOLUME	0
# define VOLNUMBER	1
# define PAGE	2

/* Change this to write output in a different order */
char *order = {"DTJVP"};
struct source {
	char journal[100];
	char year[10];
	char volume[20];
	char number[10];
	char pages[10];
};

main(argc,argv)
int argc;
char *argv[];
{
	int getaline();
	char theline[400];
	char title[10][200];
	int len,type,linetype();
	long lineno;
	int lastheader;
	int znumber;
	int ntitles;
	int nchars;
	struct source thesource;

	if(argc == 2){
		strcpy(order,argv[1]);
		if( !checkorder(order) ){
			fprintf(stderr,"usage: %s sorting_order\n",argv[0]);
			fprintf(stderr,"illegal sorting key %s\n",order);
			exit(-1);
		}
	}

	lineno = 0;
	lastheader = -1;
	znumber = 1;
	ntitles = 0;
	while((nchars = getaline(theline)) != 0){
		lineno++;
		type = linetype(theline);
		switch(type){
			case AUTHOR:
				len = strlen(theline);
				doauthors(theline,len);
				lastheader = AUTHOR;
				break;
			case TITLE:
				len = strlen(theline);
				strcpy(title[0],"%T ");
				strcat(title[0],theline+LINEBEG);
				lastheader = TITLE;
				ntitles++;
				break;
			case SOURCE:
				dosource(theline,nchars,&thesource);
				printcite(title,ntitles,thesource,order);
				lastheader = SOURCE;
				break;
			case ABSTRACT:
				printf("%%X    %s",theline+LINEBEG);
				lastheader = ABSTRACT;
				break;
			case NUMBER:
				printf("%%G %s",theline+LINEBEG);
				lastheader = NUMBER;
				break;
			case YEAR:
				break;
			case WHERE:
				printf("%%W %s",theline+LINEBEG);
				lastheader = WHERE;
				break;
			default:
				if(lastheader == AUTHOR){
					len = strlen(theline);
					doauthors(theline,len);
					lastheader = AUTHOR;
					break;
				}
				if(isnumber(theline)){
					printf("%%Z %d\n",znumber);
					znumber++;
					ntitles = 0;
					break;
				}
				if(lastheader == TITLE){
					strcpy(title[ntitles],"%T ");
					strcat(title[ntitles],theline+LINEBEG);
					ntitles++;
				} else if(lastheader == WHERE){
					printf("%%W %s",theline+LINEBEG);
				} else {
					printf("%s",theline);
					lastheader = nflags;
				}
		}
	}
}

getaline(theline)
char *theline;
{
	int c,i;
	int nchars;

	i = 0;
	while( (c = getchar()) != '\n' && c != EOF){
		if( c == '#' || c == '%'){
			theline[i++] = '\\';
		}
		theline[i++] = c;
	}
	if(i > 0 || c == '\n'){
		theline[i++] = '\n';
		theline[i++] = '\0';
		nchars = i;
		return(nchars);
	} else {
		nchars = 0;
		return(nchars);
	}
}

linetype(theline)
char *theline;
{
	int i;

	if(theline[COLONLOC] != ':') return(nflags);

	for(i = 0; i < nflags; i++){
		if(strncmp(theline,lineflags[i],COLONLOC) == 0) return(i);
	}
	return(nflags);
}
/*
The source field for medline is divided by a semicolon into
Journal Name Year month; Volume(number):pagerange
*/
dosource(theline,nchars,thesource)
char *theline;
int nchars;
struct source *thesource;
{
	int c;
	int i,j,iend;
	int semiloc;
	int lineno,yearloc,monthloc;
	char srcline[1000];
	int linelen;
	char *cp;
	int type;
	int len;
	char pagerange[30];

/* get the whole source */
	strcpy(srcline,theline+COLONLOC+2);
	linelen = nchars;
	while((nchars = getaline(theline)) != 0){
		if(theline[COLONLOC] != ':'){
			linelen += nchars;
			strcat(srcline,theline+COLONLOC+2);
		} else {
			break;
		}
	}
/* store words in the journal field until a keyword is found */
	cp = theline;
	while( (type = nextword(cp,&len))  == TEXTWORD){
		strncat(thesource->journal,len);
		cp = cp+len;
	}
/*
SOURCE:  Medical & Biological Engineering & Computing, vol.33, no.5, 
	p. 669-75

SOURCE:  Proceedings of the Annual International Conference of the 
	IEEE Engineering in Medicine and Biology Society (IEEE Cat. 
	No.88CH2566-8), p. 4 vol. (xxxiv+xxiii+xix+xix+1937), 473-4
*/
	while( (type = nextword(cp,&len)) != TEXTWORD){
		switch(type){
			case VOLUME:
				strcpy(thesource->volume,"%V ");
				type = nextword(cp,&len);
				strncat(thesource->volume,cp,len);
				break;
			case VOLNUMBER:
				strcpy(thesource->number,"%N ");
				type = nextword(cp,&len);
				strncat(thesource->number,cp,len);
				break;
			case PAGE:
				strcpy(thesource->pages,"%P ");
				if(findpages(cp,pagerange)){
				} else {
					type = nextword(cp,&len);
					strncat(thesource->pages,cp,len);
				}
				break;
			default:
				break;
		}
	}
}

findpages(cp,pagerange)
char *cp;
char *pagerange;
{
	char c;
	char *cpp;
	char *dash;
	int firstpage,lastpage;

	cpp = cp;

/* look for a '-' */
	while( (c = *cpp) != '\n' && c != '-') cpp++;
	if(c == '\n') return(FALSE);
	dash = cpp;

/* now search backwards for a blank */
	cpp--;
	while( (c = *cpp) != ' ' && cpp > cp) cpp--;

/* Try to read a page number */
	if( sscanf(cpp,"%d",&firstpage) != 1){
		return(FALSE);
	}
	if( sscanf(dash+1,"%d",&lastpage) != 1){
		return(FALSE);
	}
/* Everything checks out, copy the page range */
	while( (c = *cpp) != ' ' && c != '\n'){
		*pagerange = c;
		pagerange++;
		cpp++;
	}
	return(TRUE);
}

nextword(cp,len)
char *cp;
int *len;
{
	int i;
	int type;
	char c;
	int nchars;

	type = -1;
	while( (c = *cp) != ' ' && c != '\0'){
		cp++;
		*len = *len + 1;
	}
	for(i = 0; i < nkwds; i++){
		nchars = strlen(kwd[i]);
		if(strncmp(kwd,cp,nchars) == 0){
			type = i;
			break;
		}
	}
	return(type);
}

isnumber(theline)
char *theline;
{
	int c,i;

	i = 0;
	while( (c = theline[i++]) != '\n'){
		if(c > '9' || c < '0') return(FALSE);
	}
	if(i > 1) return(TRUE);
	else return(FALSE);
}

fillstring(strng,type,theline,istart,iend)
char *strng;
char *type;
char *theline;
int istart;
int iend;
{
	int i,j;

	sprintf(strng,"%%%s ",type);
	j = 3;
	for(i = istart; i < iend; i++){
		strng[j++] = theline[i];
	}
	strng[j++] = '\n';
	strng[j++] = '\0';
}
printcite(title,ntitles,thesource,order)
char title[][200];
int ntitles;
struct source thesource;
char *order;
{
	int norder;
	int i,j;
	int len;
	int isreview;

	norder = strlen(order);

	for(i = 0; i < norder; i++){
		switch(order[i]){
			case 'D':
				printf("%s",thesource.year);
				break;
			case 'T':
				for(j = 0; j < ntitles-1; j++){
					printf("%s",title[j]);
				}
				isreview = rmreview(title[j]);
				len = strlen(title[j]);
				if(title[j][len-2] == '.'){
					title[j][len-1] = '\0';
					title[j][len-2] = '\n';
				}
				printf("%s",title[j]);
				if(isreview){
					printf("%%Y Review\n");
				}
				break;
			case 'J':
				printf("%s",thesource.journal);
				break;
			case 'V':
				printf("%s",thesource.volume);
				break;
			case 'P':
				printf("%s",thesource.pages);
				break;
			default:
				break;
		}
	}
	
}
/* removes the string "[Review]" from the title field */
rmreview(string)
char *string;
{
	int i,len;

	len = strlen(string);

	if( strncmp(string+len-9,"[Review]",8) == 0 ){
		string[len-10] = '\n';
		string[len-9] = '\0';
		return(TRUE);
	}
	return(FALSE);
}

checkorder(order)
char *order;
{
	int norder,i,j,c;
	char keys[20],nkeys;
	char gotone;

	strcpy(keys,"DTJVP");
	nkeys = strlen(keys);
	norder = strlen(order);
	for(i = 0; i < norder; i++){
		c = order[i];
		gotone = FALSE;
		for(j = 0; j < nkeys; j++){
			if( c == keys[j]){
				gotone = TRUE;
				break;
			}
		}
		if(!gotone) return(FALSE);
	}
	return(TRUE);
}
doauthors(theline,len)
char *theline;
int len;
{
	int c;
	int i;

	printf("%%A ");
	for(i = len-1; i > 0; i--){
		if(theline[i] != '\n' && theline[i] != ' ') break;
	}
	len = i;
	if(theline[len] != ';') len++;
	for(i = COLONLOC+3; i < len; i++){
/* if theline[len-1] is a ';', there will be more authors coming later */
		if( (c = theline[i]) == ';' && i != len-1){
			printf("\n%%A ");
			i++;
		} else if(i == len-1){
			putchar(theline[i]);
		}else{
			putchar(c);
		}
	}
	putchar('\n');
}

/*
 
------------------------------------------------------------ 
PLEASE DO NOT REPLY OR SEND MESSAGES TO THIS EMAIL ADDRESS. 
------------------------------------------------------------ 

   RECORD NO.:  3399011 INSPEC Abstract No: A89078449
       AUTHOR:  Gilson, W.; Orphanoudakis, S.
       EDITOR:  Harris, G.; Walker, C.
  CORP SOURCE:  Lincoln Lab., MIT, Lexington, MA, USA
        TITLE:  Error bounds for wide-band high resolution Doppler 
                ultrasound blood flow measurement
       SOURCE:  Proceedings of the Annual International Conference of the 
                IEEE Engineering in Medicine and Biology Society (IEEE Cat. 
                No.88CH2566-8), p. 4 vol. (xxxiv+xxiii+xix+xix+1937), 473-4
PLACE OF PUBL:  USA
     LANGUAGE:  English
    PUBLISHER:  IEEE; New York, NY, USA
  SPONSOR ORG:  IEEE
CONF LOCATION:  New Orleans, LA, USA; 4-7 Nov. 1988
         YEAR:  1988
 COPYRIGHT NO:  CH2566-8/88/0000-0473$01.00
    TREATMENT:  T Theoretical or Mathematical
     ABSTRACT:  Cramer-Rao bounds on the RMS error of Doppler flow 
                measurement instruments are presented for CW Doppler, 
                narrowband pulsed Doppler, and wideband pulsed Doppler blood 
                flow measurement for both optimal and conventional 
                receivers. The bounds indicate that for a given SNR (signal-
                to-noise ratio), narrowband pulsed Doppler can be as 
                accurate as CW Doppler. They also suggest that with an 
                optimal estimator the RMS flow velocity error of wideband 
                pulsed Doppler varies slightly from that of CW Doppler while 
                with a conventional receiver the RMS error increases rapidly 
                with increasing pulse bandwidth (5 Refs.)
  DESCRIPTORS:  biomedical measurement; biomedical ultrasonics; Doppler 
                effect; flow measurement; haemodynamics; measurement errors
  IDENTIFIERS:  Doppler ultrasound blood flow measurement; Cramer-Rao 
                bounds; RMS error; CW Doppler; narrowband pulsed Doppler; 
                wideband pulsed Doppler; signal-to-noise ratio
  CLASS CODES:  A8760B (Sonic and ultrasonic radiation); A8770E (Diagnostic 
                methods and instrumentation); A8745H (Haemodynamics, 
                pneumodynamics)

*/ 
