# include <math.h>
# define PI 3.14159265358979323844
# define TRUE		1
# define FALSE		0

void  realft(data,n,isign)
float *data;
int n,isign;
/*
This routine is from "Numerical Recipes in C", copyright 1988 by
Cambridge University Press.

It calculates the Fourier transform of a set of 2n real-valued data points.
It replaces this data (which is stored in the array "data[1..2n]") by
the positive frequency half of its complex Fourier Transform.  Note
that the first and last components are handled specially since they
are real-valued.  They are
returned as elements data[1] and data[2] respectively.  n must be a
power of 2.  This routine also calculates the inverse transform of a
complex data array if it is the transform of real data.  Results in
this case must be multiplied by 1/n.  If isign is 1, the forward
transform is computed.  Otherwise the inverse transform is computed.
*/
{
	int i,i1,i2,i3,i4,n2p3;
	float c1=0.5,c2,h1r,h1i,h2r,h2i;
	double wr,wi,wpr,wpi,wtemp,theta; /* double for the trig recurrences */
	int stat;
	int four1();

	theta = PI/(double) n;    /* initialize the recurrence */
	if(isign == 1){
		c2 = -0.5;
		stat = four1(data,n,1);
	} else {
		c2 = 0.5;
		theta = -theta;
	}
	wtemp = sin(0.5*theta);
	wpr = -2.0*wtemp*wtemp;
	wpi = sin(theta);
	wr = 1.0 + wpr;
	wi = wpi;
	n2p3 = 2*n + 3;
	for(i = 2; i <= n/2; i++){  /* case i = 1 done separately */
		i4 = 1 + (i3=n2p3-(i2=1+(i1=i+i-1)));
		h1r = c1*(data[i1]+data[i3]);
		h1i = c1*(data[i2]-data[i4]);
		h2r = -c2*(data[i2]+data[i4]);
		h2i = c2*(data[i1]-data[i3]);
		data[i1] = h1r + wr*h2r - wi*h2i;
		data[i2] = h1i+wr*h2i + wi*h2r;
		data[i3] = h1r - wr*h2r + wi*h2i;
		data[i4] = -h1i + wr*h2i + wi*h2r;
		wr = (wtemp=wr)*wpr - wi*wpi + wr;
		wi = wi*wpr + wtemp*wpi + wi;
	}
	if(isign == 1){
		data[1] = (h1r=data[1]) + data[2];
		data[2] = h1r - data[2];
	} else {
		data[1] = c1*((h1r=data[1]) + data[2]);
		data[2] = c1*(h1r-data[2]);
		stat = four1(data,n,-1);
	}
}
